unit Lib_GridEx;

interface
uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Graphics,

  // To sort by TFDDataset
  FireDAC.Comp.DataSet,
  Data.DB,


  FMXTee.Control,
  FMXTee.Grid,

  // TVirtualData
  Tee.GridData,
  // TVirtualDBData
  Tee.GridData.DB,

  Tee.GridData.Rtti,
  Tee.Grid.Columns,
  Tee.Grid.Header;



  type
    TVirtualFDDatasetData = class(TVirtualDBData)
  private
    type
      TLastSorted = record
    public
      Column : TColumn;
      SortState :TSortState;
    end;

  private
    FlLastSortedColumn : TLastSorted;
    function GetSortableDataSet: TFDDataSet;
  public

    function CanSortBy(const AColumn:TColumn):Boolean; override;
    procedure SortBy(const AColumn:TColumn); override;
    function IsSorted(const AColumn:TColumn; out Ascending:Boolean):Boolean; override;
    class function From(const ASource: TComponent): TVirtualData;

  end;


  type
    TDataGrid = class(TTeeGrid)
  private
  public
    { Public declarations }
    constructor create(AOwner:TComponent);override;
  end;

{
  Implementar lo siguiente
  Permitir ordenar (Los datos del grid, no los datos del servidor)
  Permitir controles customizados en los grid




Virtual Mode
C:\Program Files (x86)\Steema Software\Steema TeeGrid for VCL & FMX Registered version-1.04\demos\VirtualData\Virtual Mode\Unit_VirtualMode.pas

Cell Editors
C:\Program Files (x86)\Steema Software\Steema TeeGrid for VCL & FMX Registered version-1.04\demos\FireMonkey\Cell Editors\Unit_Editors.pas

Custom Sorting
C:\Program Files (x86)\Steema Software\Steema TeeGrid for VCL & FMX Registered version-1.04\demos\FireMonkey\Custom Sorting\Unit_Custom_Sorting.pas


Row_SubTitles
C:\Program Files (x86)\Steema Software\Steema TeeGrid for VCL & FMX Registered version-1.04\demos\FireMonkey\Row_SubTitles\Unit_Row_SubTitles.pas

Detail rows

}



implementation

{ TDataGrid }



constructor TDataGrid.create(AOwner: TComponent);
  function CreateSortable:TSortableHeader;
  begin
    result:=TSortableHeader.Create(Self.Header.Changed);
    // cosmetic example
    result.Format.Brush.Color:=TColors.Green;
    result.Format.Stroke.Show;
  end;
begin

  inherited Create(AOwner);
  // Allow sort if Data is TVirtualFDDatasetData
  Header.Sortable := True;
  Header.SortRender :=  CreateSortable();

end;



{ TVirtualFDDatasetData }

class function TVirtualFDDatasetData.From(const ASource: TComponent):TVirtualData;

  function From(const ASource:TDataSource):TVirtualFDDatasetData;
  begin
    result:=TVirtualFDDatasetData.Create;
    result.IDataSource:=ASource;
    result.IDataSet:=ASource.DataSet;
    result.CreateLink;
  end;

  function FromDataSet(const ASource:TDataSet):TVirtualFDDatasetData;
  var tmp : TDataSource;
  begin
    tmp:=TDataSource.Create(nil);
    tmp.DataSet:=ASource;

    result:=From(tmp);
    result.OwnsDataSource:=True; // <-- to remember to destroy result.IDataSource
  end;

begin

  if ASource is TDataSet then
     result:=FromDataSet(ASource as TDataSet)
  else
  if ASource is TDataSource then
     result:=From(ASource as TDataSource)
  else
     result:=nil;

end;


function TVirtualFDDatasetData.CanSortBy(const AColumn: TColumn): Boolean;
begin

 result := GetSortableDataSet() <> nil;
 if result then
   result:= (AColumn <> nil);

end;

function TVirtualFDDatasetData.GetSortableDataSet: TFDDataSet;
begin

  result := nil;
  if Self.DataSet is TFDDataSet then
    result := TFDDataSet(Self.DataSet);

end;

  function _NextSortState(CurrentSortState:TSortState):TSortState;
  begin
    case CurrentSortState of
      None      : result := TSortState.Ascending ;
      Ascending : result := TSortState.Descending;
      Descending: result := TSortState.None;
    end;
  end;

function TVirtualFDDatasetData.IsSorted(const AColumn: TColumn; out Ascending: Boolean): Boolean;
begin

   if AColumn = FlLastSortedColumn.Column then begin
     Ascending := FlLastSortedColumn.SortState = TSortState.Ascending;
     result := FlLastSortedColumn.SortState <> TSortState.None;
   end else
     result := False;

end;

procedure TVirtualFDDatasetData.SortBy(const AColumn: TColumn);

  procedure _SortDataSet(SortAction :TSortState);
  begin
      //Self.BeginUpdate;
      //SortableDataSet.DisableControls;
      try
        case SortAction of
          None       : GetSortableDataSet().IndexFieldNames := '';
          Ascending  : GetSortableDataSet().IndexFieldNames := AColumn.Header.Text+':A';
          Descending : GetSortableDataSet().IndexFieldNames := AColumn.Header.Text+':D';
        end;

      finally
        //SortableDataSet.EnableControls;
        //Self.EndUpdate;
      end;
  end;

begin

  // Is TDataSet desdendant Sortable?
  if GetSortableDataSet() <> nil then begin

    // If the last ordered column is not the current one
    if AColumn <> FlLastSortedColumn.Column then begin
       // Remember last sorted column and next SortState from none
       FlLastSortedColumn.Column    := AColumn;
       FlLastSortedColumn.SortState := _NextSortState(TSortState.None);
    end else begin
       // Next Sort state
       FlLastSortedColumn.SortState := _NextSortState(FlLastSortedColumn.SortState);
    end;

    // Try sort TFDDataSet
    _SortDataSet(FlLastSortedColumn.SortState);

  end else begin
     FlLastSortedColumn.Column    := nil;
     FlLastSortedColumn.SortState := TSortState.None
  end;


end;

end.
